package net.sergeych.kiloparsec

import kotlinx.serialization.KSerializer
import kotlinx.serialization.serializer
import kotlin.reflect.KProperty

/**
 * delegate returning function that creates a [Command] in the current context which by default has the name of
 * the property.
 */
inline fun <reified A, reified R> command(overrideName: String? = null): CommandDelegate<A, R> {
    return CommandDelegate(
        serializer<A>(),
        serializer<R>(),
        overrideName
    )
}

/**
 * Delegate to create [Command] via property
 */
class CommandDelegate<A, R>(
    private val argsSerializer: KSerializer<A>,
    private val resultSerializer: KSerializer<R>,
    private val overrideName: String? = null
) {
    operator fun getValue(nothing: Nothing?, property: KProperty<*>): Command<A, R> {
        return Command(
            overrideName ?: property.name,
            argsSerializer,
            resultSerializer
        )
    }
}