package net.sergeych.kiloparsec

import kotlinx.coroutines.CompletableDeferred
import net.sergeych.mp_logger.LogTag
import net.sergeych.mp_logger.Loggable
import net.sergeych.utools.unpack

private var L1IdCounter = 0

// todo: We don't need it deferred here
class KiloRemoteInterface<S>(
    private val deferredParams: CompletableDeferred<KiloParams<S>>,
    private val clientInterface: LocalInterface<KiloScope<S>>,
) : RemoteInterface, Loggable by LogTag("L1TR:${++L1IdCounter}") {

    override suspend fun <A, R> call(cmd: Command<A, R>, args: A): R {
        val params = deferredParams.await()
        val block: Transport.Block = unpack(
            params.decrypt(
                params.transport.call(L0Call, params.encrypt(cmd.packCall(args)))
            )
        )
        return when (block) {
            is Transport.Block.Response -> {
                cmd.unpackResult(block.packedResult)
            }

            is Transport.Block.Error -> clientInterface.decodeAndThrow(block)
            else -> throw RemoteInterface.Exception("unexpected block type: $block")
        }
    }
}

