package net.sergeych.kiloparsec.adapter

import kotlinx.coroutines.channels.ReceiveChannel
import kotlinx.coroutines.flow.Flow

/**
 * Multiplatform implementation of an internet address.
 * Notice to implementors. It must provide correct and effective [equals] and [hashCode].
 */
interface NetworkAddress {
    val host: String
    val port: Int
}

/**
 * Multiplatform datagram abstraction
 */
interface Datagram {
    /**
     * Received message
     */
    val message: UByteArray

    /**
     * Address from where the message was sent
     */
    val address: NetworkAddress
}

@OptIn(ExperimentalStdlibApi::class)
interface DatagramConnector: AutoCloseable {

    val incoming: ReceiveChannel<Datagram>
    suspend fun send(message: UByteArray, networkAddress: NetworkAddress)
    @Suppress("unused")
    suspend fun send(message: UByteArray, datagramAddress: String) {
        send(message, datagramAddress.toNetworkAddress())
    }

    suspend fun send(message: UByteArray,host: String,port: Int) =
        send(message, NetworkAddress(host,port))
    override fun close()
}

expect fun NetworkAddress(host: String,port: Int): NetworkAddress

fun String.toNetworkAddress() : NetworkAddress {
    val (host, port) = this.split(":").map { it.trim()}
    return NetworkAddress(host, port.toInt())
}

expect fun acceptTcpDevice(port: Int): Flow<InetTransportDevice>

expect suspend fun connectTcpDevice(address: NetworkAddress): InetTransportDevice