package net.sergeych.kiloparsec.adapters

import com.ionspin.kotlin.crypto.util.decodeFromUByteArray
import com.ionspin.kotlin.crypto.util.encodeToUByteArray
import kotlinx.coroutines.*
import kotlinx.coroutines.test.runTest
import net.sergeych.kiloparsec.adapter.UdpServer
import net.sergeych.kiloparsec.adapter.acceptTcpDevice
import net.sergeych.kiloparsec.adapter.connectTcpDevice
import net.sergeych.kiloparsec.adapter.toNetworkAddress
import net.sergeych.mp_logger.Log
import net.sergeych.tools.ProtectedOp
import org.junit.jupiter.api.Assertions.assertEquals
import kotlin.test.Test
import kotlin.test.assertContains

class NetworkTest {

    @Test
    fun udpProviderTest() = runTest {
        Log.connectConsole(Log.Level.DEBUG)
        val s1 = UdpServer(17120)
        val s2 = UdpServer(17121)
        s1.send("Hello".encodeToUByteArray(), "localhost", 17121)
        val d1 = s2.incoming.receive()
        assertEquals(d1.address.port, 17120)
        assertEquals("Hello", d1.message.toByteArray().decodeToString())
        s1.send("world".encodeToUByteArray(), d1.address)
        assertEquals("world", s1.incoming.receive().message.toByteArray().decodeToString())
        //        println("s1: ${s1.bindAddress()}")

    }

    @Test
    fun tcpAsyncConnectionTest() = runTest {
        Log.connectConsole(Log.Level.DEBUG)

        coroutineScope {
            val serverFlow = acceptTcpDevice(17171)
            val op = ProtectedOp()
            var pills = setOf<String>()
            val j = launch {
                serverFlow.collect { device ->
                    launch {
                        device.output.send("Hello, world!".encodeToUByteArray())
                        device.output.send("Great".encodeToUByteArray())
                        while (true) {
                            val x = device.input.receive()?.decodeFromUByteArray() ?: break
                            if (x.startsWith("die")) {
                                op {
                                    pills += x
                                }
                                cancel()
                            }
                            else
                                println("ignoring unexpected input: $x")
                        }
                    }
                }
            }
            yield()
            run {
                val s = connectTcpDevice("127.0.1.1:17171".toNetworkAddress())
                assertEquals("Hello, world!", s.input.receive()!!.decodeFromUByteArray())
                assertEquals("Great", s.input.receive()!!.decodeFromUByteArray())
                s.output.send("Goodbye".encodeToUByteArray())
                s.output.send("die1".encodeToUByteArray())
                s.close()
            }
            val s1 = connectTcpDevice("127.0.1.1:17171".toNetworkAddress())
            assertEquals("Hello, world!", s1.input.receive()!!.decodeFromUByteArray())
            assertEquals("Great", s1.input.receive()!!.decodeFromUByteArray())
            s1.output.send("die2".encodeToUByteArray())
            s1.close()

            // check that channels were flushed prior to closed:
            assertContains(pills, "die1")
            assertContains(pills, "die2")

            // Check that server jobs are closed
            j.cancelAndJoin()
        }
    }
}